#include "includes.h"

const struct s_Demo Demo[]=
{
  {"RTC",RTC_Demo},
  {"USB OTG Connector",USBOTG_Demo},
  {"USB HOST Connector",USBHOST_Demo},
  {"SD Card",SDCard_Demo},
};

void Blink (int Delay)
{
  STM_EVAL_LEDInit (LED1);
  STM_EVAL_PBInit (BUTTON_WAKEUP, BUTTON_MODE_GPIO);

  STM_EVAL_LEDOn (LED1);
  DelayResolution100us (Delay);
  STM_EVAL_LEDOff (LED1);
  DelayResolution100us (Delay);
}

int RTC_Demo(void)
{
  int x=0;
 
  /*SysTick to times per second*/
  SysTickStart(1);

  /* Enable LSE */
  RCC_APB1PeriphClockCmd(  RCC_APB1Periph_PWR, ENABLE);
  PWR_BackupAccessCmd(ENABLE);
  /* */
  RCC_LSEConfig(RCC_LSE_ON);
  /* Wait till LSE is ready */

  /* Select LSE as RTC Clock Source */
  RCC_RTCCLKConfig(RCC_RTCCLKSource_LSE);

  /* Enable RTC Clock */
  RCC_RTCCLKCmd(ENABLE);
  
  while (!STM_EVAL_PBGetState (BUTTON_WAKEUP))
  {
    /*Check Ticks*/
    if( SysTick->CTRL & (1<<16))
    {
      if (x)
        STM_EVAL_LEDOn (LED1);
      else
        STM_EVAL_LEDOff (LED1);
      x = !x;
    }
  }
  while (STM_EVAL_PBGetState (BUTTON_WAKEUP));

  return 0;
}

void USB_USR_Null(void)
{
  
}

void USB_USR_Reset(Int8U speed)
{
  
}

USBD_Usr_cb_TypeDef USR_cb =
{
  USB_USR_Null,
  USB_USR_Reset,
  USB_USR_Null,
  USB_USR_Null,
  USB_USR_Null,
  USB_USR_Null,
  USB_USR_Null,  
};
#define MOUSE_DELTA           4
#ifdef USB_OTG_HS_INTERNAL_DMA_ENABLED
  #if defined ( __ICCARM__ ) /*!< IAR Compiler */
    #pragma data_alignment=4   
  #endif
#endif /* USB_OTG_HS_INTERNAL_DMA_ENABLED */
__ALIGN_BEGIN USB_OTG_CORE_HANDLE  USB_OTG_dev __ALIGN_END;

void USB_OTG_BSP_Init(USB_OTG_CORE_HANDLE *pdev)
{
}

void USB_OTG_BSP_EnableInterrupt(USB_OTG_CORE_HANDLE *pdev)
{
}

void USB_Mouse(void)
{

Int8U buf[4];

#define Buttons buf[0]
#define X  buf[1]
#define Y  buf[2]
int D=0, Times=0;
Int8U ButtonsHold = 0;

  ENTR_CRT_SECTION();
  // SysTick end of count event each 0.01s 
  /*SysTick to times per second*/
  SysTickStart(100);
  
  EXT_CRT_SECTION();

  /*Init Buttons*/
  STM_EVAL_PBInit(BUTTON_WAKEUP, BUTTON_MODE_GPIO);

  X = 0;
  Y = 0;
  Buttons = 0;
  ButtonsHold = 0;
  
  while (!STM_EVAL_PBGetState (BUTTON_WAKEUP))
  {
      switch (D)
      {
      case 0:
        X += MOUSE_DELTA;
        break;
      case 1:
        Y += MOUSE_DELTA;
        break;
      case 2:
        X -= MOUSE_DELTA;
        break;
      case 3:
        Y -= MOUSE_DELTA;
        break;
      }
      
      Times++;
      D = (D + (Times / 20)) % 4;
      Times = Times % 20;

        if(Y || X  || Buttons || (ButtonsHold ^ Buttons))
        {
          // Send report
          USBD_HID_SendReport (&USB_OTG_dev, 
                         buf,
                         4);
          ButtonsHold = Buttons;
          Y = X = Buttons = 0;
        }
  }

  while (STM_EVAL_PBGetState (BUTTON_WAKEUP));
}

int USBOTG_Demo(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;    
  NVIC_InitTypeDef NVIC_InitStructure; 

  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOB , ENABLE);
  
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12 | 
                                GPIO_Pin_14 | 
                                GPIO_Pin_15;
  
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_Init(GPIOB, &GPIO_InitStructure);  
  
  GPIO_PinAFConfig(GPIOB,GPIO_PinSource12, GPIO_AF_OTG2_FS) ; 
  GPIO_PinAFConfig(GPIOB,GPIO_PinSource14,GPIO_AF_OTG2_FS) ; 
  GPIO_PinAFConfig(GPIOB,GPIO_PinSource15,GPIO_AF_OTG2_FS) ;
  
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_13;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_Init(GPIOB, &GPIO_InitStructure);  

  
  RCC_AHB1PeriphClockCmd( RCC_AHB1Periph_OTG_HS, ENABLE) ;  
    
  USBD_Init(&USB_OTG_dev,
            USB_OTG_HS_CORE_ID,
            &USR_desc, 
            &USBD_HID_cb, 
            &USR_cb);
  
  NVIC_PriorityGroupConfig(NVIC_PriorityGroup_1);

  NVIC_InitStructure.NVIC_IRQChannel = OTG_HS_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 3;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);  

  NVIC_PriorityGroupConfig(NVIC_PriorityGroup_1);
  NVIC_InitStructure.NVIC_IRQChannel = OTG_HS_EP1_OUT_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 2;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);  
  
  NVIC_PriorityGroupConfig(NVIC_PriorityGroup_1);
  NVIC_InitStructure.NVIC_IRQChannel = OTG_HS_EP1_IN_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);   

  USB_Mouse();

  return 0;
}

int USBHOST_Demo(void)
{
GPIO_InitTypeDef GPIO_InitStructure;    
NVIC_InitTypeDef NVIC_InitStructure; 

  RCC_AHB1PeriphClockCmd( RCC_AHB1Periph_GPIOA , ENABLE);  
  
  /* Configure SOF ID DM DP Pins */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8  | 
                                GPIO_Pin_11 | 
                                GPIO_Pin_12;
  
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL ;
  GPIO_Init(GPIOA, &GPIO_InitStructure);  
  
  GPIO_PinAFConfig(GPIOA,GPIO_PinSource8,GPIO_AF_OTG1_FS) ;
  GPIO_PinAFConfig(GPIOA,GPIO_PinSource11,GPIO_AF_OTG1_FS) ; 
  GPIO_PinAFConfig(GPIOA,GPIO_PinSource12,GPIO_AF_OTG1_FS) ;
  
  /* Configure  VBUS Pin */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_OD;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL ;
  GPIO_Init(GPIOA, &GPIO_InitStructure);    
  
  /* Configure ID pin */
  GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_10;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_OD;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP ;  
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_Init(GPIOA, &GPIO_InitStructure);  
  GPIO_PinAFConfig(GPIOA,GPIO_PinSource10,GPIO_AF_OTG1_FS) ;   

  RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);
  RCC_AHB2PeriphClockCmd(RCC_AHB2Periph_OTG_FS, ENABLE) ; 


  USBD_Init(&USB_OTG_dev,
            USB_OTG_FS_CORE_ID,
            &USR_desc, 
            &USBD_HID_cb, 
            &USR_cb);
  
  NVIC_PriorityGroupConfig(NVIC_PriorityGroup_1);
  NVIC_InitStructure.NVIC_IRQChannel = OTG_FS_IRQn;  
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 3;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);  
  
  USB_Mouse();
  
  return 0;
}


#define UPDATE_SHOW_DLY       ((Int32U)(0.5 * 2))

int SDCard_Demo(void)
{
int ret = 1, DemoOver = 0, Delay=100;
Int32U Dly = UPDATE_SHOW_DLY;
DiskStatusCode_t StatusHold = (DiskStatusCode_t) -1;
Int32U Size,Tmp;
Int8U buff[SD_DEF_BLK_SIZE];

//  printf("Press Esc for exit\n\r");
  /*SysTick to times per second*/
  SysTickStart(2);
  //
  SdDiskInit();

  __enable_interrupt();

  while (1)
  {
    if (DemoOver)
      break;
    if( SysTick->CTRL & (1<<16))
    {
      // Update MMC/SD card status
      SdStatusUpdate();
      if(Dly-- == 0)
      {
        // LCD show
        Dly = UPDATE_SHOW_DLY;
        // Current state of MMC/SD show
        pDiskCtrlBlk_t pMMCDiskCtrlBlk = SdGetDiskCtrlBkl();
        if(StatusHold != pMMCDiskCtrlBlk->DiskStatus)
        {
          StatusHold = pMMCDiskCtrlBlk->DiskStatus;
          switch (pMMCDiskCtrlBlk->DiskStatus)
          {
          case DiskCommandPass:
            // Calculate MMC/SD size [MB]
            Size = (pMMCDiskCtrlBlk->BlockNumb * pMMCDiskCtrlBlk->BlockSize);
            Tmp  = Size/1000000;
            Tmp += ((Size%1000000)<1000000/2)?0:1;
            if(pMMCDiskCtrlBlk->DiskType == DiskMMC)
            {
              //printf("MMC Card - %d MB\n\r",Tmp);
              Delay = 1000;
            }
            else
            {
              //printf("SD Card - %d MB\n\r",Tmp);
              Delay = 1000;
            }

            if(pMMCDiskCtrlBlk->WriteProtect == TRUE)
            {
              //printf("Card is Write Protected\n\r");
              Delay = 300;
            }

            for(volatile int i = 0; 100000 > i ; i++);
            if(DiskCommandPass == SdDiskIO(buff,pMMCDiskCtrlBlk->BlockNumb-1,1,DiskRead))
            {
              if(0xAA55A55A != *((Int32U *)buff))
              {
                if(FALSE == pMMCDiskCtrlBlk->WriteProtect)
                {
                  *((Int32U *)buff) = 0xAA55A55A;
                  
                  if(DiskCommandPass != SdDiskIO(buff,pMMCDiskCtrlBlk->BlockNumb-1,1,DiskWrite) &&
                     DiskCommandPass != SdDiskIO(buff,pMMCDiskCtrlBlk->BlockNumb-1,1,DiskVerify) )
                    {
                      //printf("Card I/O error\n\r");
                      Delay = 100;
                      ret = 1;
                      DemoOver = 1;
                    }
                    else
                    {
                      ret = 0;
                      DemoOver = 1;
                    }
                }
              }
              else
              {
                ret = 0;
                DemoOver = 1;
              }
            }
            else
            {
              //printf("Card I/O error\n\r");
              Delay = 100;
              ret = 1;
              DemoOver = 1;
            }
            break;
          default:
            //printf("Pls, Insert Card\n\r");
            HardwareInit();
            STM_EVAL_LEDOn (LED1);
          }
        }
      }
    }
  }

  __disable_interrupt();

  HardwareInit ();
  while (!STM_EVAL_PBGetState (BUTTON_WAKEUP))
    Blink (Delay);
  while (STM_EVAL_PBGetState (BUTTON_WAKEUP));

  return ret;
}